/* This is Lexical specification file.
 * Eventhough I used the grammar from 
 * www.home.attbi.com/~bronnikov/java.html
 * I had done little modifications to the grammar.
 * 1) Changed the regular expression used for Comment1
 * 2) Added new HexEscape
 * 3) Added '$' in identifier definitions.
 */
/**
 * @Developed by N V Sreenivasa Rao
 * @Supervisor   Prof Pankaj Jalote
 * @URL          http://www.cse.iitk.ac.in/users/jalote/download/javacoverage/index.html
 * @title        JavaCoverage
 * @version      1.0
 */
%{
#include "flags.h"
#include "error.h"
#include "statement_count_list.h"
#include "token.h"
#include "block_list.h"
#include "y.tab.h"
#include <string.h>
#include <unistd.h>

void before_control_flow_statements(int);
int lineno = 1;
int fd;
%}

%e 1600
%n 800
%p 5000

Separator	[\(\)\{\}\[\]\;\,\.]
Delimiter1	[\=\>\<\!\~\?\:\+\-\*\/\&\|\^\%]
HexDigit	[0-9a-fA-F]
Digit		[0-9]
OctalDigit	[0-7]
TetraDigit	[0-3]
NonZeroDigit	[1-9]
Letter		[a-zA-Z_$]
AnyButSlash	[^\/]
AnyButAstr	[^\*]
BLANK		[ ]
BLK		[\b]
TAB		[\t]
FF		[\f]
ESCCHR		[\\]
CR		[\r]
LF		[\n]
UniEsc          [\1b]

OctEscape1	[\\]{OctalDigit}
OctEscape2	[\\]{OctalDigit}{OctalDigit}
OctEscape3	[\\]{TetraDigit}{OctalDigit}{OctalDigit}
OctEscape	({OctEscape1}|{OctEscape2}|{OctEscape3})

HexEscape       [\\][u]{HexDigit}{HexDigit}{HexDigit}{HexDigit}

Escape		[\\]([r]|[n]|[b]|[f]|[t]|[\\]|[\']|[\"])
ULetter         ({Letter}|{UniEsc})
Identifier 	{ULetter}({ULetter}|{Digit})*

Comment1        [\/][\*](([^\*]*[\*]+)([^\/\*][^\*]*[\*]+)*[\/])
Comment2        [\/][\/].*
Comment		({Comment1}|{Comment2})

Dimension	[\[]({CR}|{LF}|{FF}|{TAB}|{BLK}|{BLANK}|{Comment})*[\]]

IntSuffix	([l]|[L])
DecimalNum	{NonZeroDigit}{Digit}*{IntSuffix}?
OctalNum	[0]{OctalDigit}*{IntSuffix}?
HexNum		[0]([x]|[X]){HexDigit}{HexDigit}*{IntSuffix}?
IntegerLiteral	({DecimalNum}|{OctalNum}|{HexNum})

Sign		([\+]|[\-])
FlSuffix	([f]|[F]|[d]|[D])
SignedInt	{Sign}?{Digit}+
Expo		([e]|[E])
ExponentPart	{Expo}{SignedInt}?
Float1          {Digit}+[\.]{Digit}+?{ExponentPart}?{FlSuffix}?
Float2		[\.]{Digit}+{ExponentPart}?{FlSuffix}?
Float3		{Digit}+{ExponentPart}{FlSuffix}?
Float4		{Digit}+{FlSuffix}
FloatingPoint	({Float1}|{Float2}|{Float3}|{Float4})

AnyChrChr	[^\\']
AnyStrChr	[^\\"]
Character	[\']({Escape}|{OctEscape}|{HexEscape}|{AnyChrChr})[\']
String		[\"]({Escape}|{OctEscape}|{AnyStrChr})*[\"]
Numeric  	({IntegerLiteral}|{FloatingPoint})
Literal		({Numeric}|{Character}|{String})

xSpace		({CR}|{LF}|{FF}|{TAB})*
xExit		("System"{xSpace}"."{xSpace}"exit"{xSpace}"("{xSpace}{IntegerLiteral}{xSpace}")"{xSpace}";")

%%


"/#"          { int state = 0;
                  char ch;
		  printf("I entered here\n");
		  while(1)   {
		  	ch = input();
		  	if(ch == '#' && state == 0)
				state = 1;
			else if(ch == '/' && state == 1)  {
				state = 2;
				break;
			} else if(ch == '\n')   {
				statement_count_data *e;
		 		e = allocate_one_element(statement_count_data);
				e->lineno = lineno;				 						    e->count = statement_counter;
		 		stat_count_list = insert_statement_count(stat_count_list, e);
		 		lineno++;
				statement_counter = 0;
				write(fd, "\n", 1);
			}
			else
				write(fd, &ch, 1);	
		  }
		}
"true"		{write(fd, yytext, yyleng);return BOOLLIT;}
"false"		{write(fd, yytext, yyleng);return BOOLLIT;}

{Separator}	{write(fd, yytext, yyleng);return yytext[0];}
{Delimiter1}	{write(fd, yytext, yyleng);return yytext[0];}
{Dimension}	{write(fd, yytext, yyleng);return OP_DIM;}

"=="		{write(fd, yytext, yyleng);return OP_EQ;}
"<="		{write(fd, yytext, yyleng);return OP_LE;}
">="		{write(fd, yytext, yyleng);return OP_GE;}
"!="		{write(fd, yytext, yyleng);return OP_NE;}
"||"		{write(fd, yytext, yyleng);return OP_LOR;}
"&&"		{write(fd, yytext, yyleng);return OP_LAND;}
"++"		{write(fd, yytext, yyleng);return OP_INC;}
"--"		{write(fd, yytext, yyleng);return OP_DEC;}
">>"		{write(fd, yytext, yyleng);return OP_SHR;}
"<<"		{write(fd, yytext, yyleng);return OP_SHL;}
">>>"		{write(fd, yytext, yyleng);return OP_SHRR;}
"+="		{write(fd, yytext, yyleng);return ASS_ADD;}
"-="		{write(fd, yytext, yyleng);return ASS_SUB;}
"*="		{write(fd, yytext, yyleng);return ASS_MUL;}
"/="		{write(fd, yytext, yyleng);return ASS_DIV;}
"&="		{write(fd, yytext, yyleng);return ASS_AND;}
"|="		{write(fd, yytext, yyleng);return ASS_OR;}
"^="		{write(fd, yytext, yyleng);return ASS_XOR;}
"%="		{write(fd, yytext, yyleng);return ASS_MOD;}
"<<="		{write(fd, yytext, yyleng);return ASS_SHL;}
">>="		{write(fd, yytext, yyleng);return ASS_SHR;}
">>>="		{write(fd, yytext, yyleng);return ASS_SHRR;}

"abstract"	{write(fd, yytext, yyleng);return ABSTRACT;}
"do"            {write(fd, yytext, yyleng);return DO;}
"implements"    {write(fd, yytext, yyleng);return IMPLEMENTS;}
"package"	{write(fd, yytext, yyleng);return PACKAGE;}
"throw"		{write(fd, yytext, yyleng);return THROW;}
"boolean"	{write(fd, yytext, yyleng);return BOOLEAN;}
"double"	{write(fd, yytext, yyleng);return DOUBLE;}
"import"	{write(fd, yytext, yyleng);return IMPORT;}
"private"	{write(fd, yytext, yyleng);return PRIVATE;}
"throws"	{write(fd, yytext, yyleng);return THROWS;}
"break"		{
		 before_control_flow_statements(0);
		 write(fd, yytext, yyleng);return BREAK;
		}
"else"		{write(fd, yytext, yyleng);return ELSE;}
"inner"		{write(fd, yytext, yyleng);return INNER;}
"protected"	{write(fd, yytext, yyleng);return PROTECTED;}
"transient"	{write(fd, yytext, yyleng);return TRANSIENT;}
"byte"		{write(fd, yytext, yyleng);return BYTE;}
"extends"	{write(fd, yytext, yyleng);return EXTENDS;}
"instanceof"	{write(fd, yytext, yyleng);return INSTANCEOF;}
"public"	{write(fd, yytext, yyleng);return PUBLIC;}
"try"		{write(fd, yytext, yyleng);return TRY;}
"case"		{before_control_flow_statements(0);
	    	 write(fd, yytext, yyleng);
		 yylval.s_str1 = allocate_one_element(struct_def1);
		 check_mem_alloc_error(yylval.s_str1);
		 yylval.s_str1->lineno = lineno;
		 return CASE;}
"final"		{write(fd, yytext, yyleng);return FINAL;}
"int"		{write(fd, yytext, yyleng);return INT;}
"rest"		{write(fd, yytext, yyleng);return REST;}
"var"		{write(fd, yytext, yyleng);return VAR;}
"cast"		{write(fd, yytext, yyleng);return CAST;}
"finally"	{write(fd, yytext, yyleng);
		 yylval.s_str1 = allocate_one_element(struct_def1);
		 check_mem_alloc_error(yylval.s_str1);
		 yylval.s_str1->lineno = lineno;
		 start_lineno = lineno;
		 return FINALLY;}
"interface"	{write(fd, yytext, yyleng);return INTERFACE;}
"return"	{
		 before_control_flow_statements(0);
		 write(fd, yytext, yyleng);
		 return RETURN;}
"void"		{write(fd, yytext, yyleng);return VOID;}
"catch"		{write(fd, yytext, yyleng);
		 yylval.s_str1 = allocate_one_element(struct_def1);
		 check_mem_alloc_error(yylval.s_str1);
		 yylval.s_str1->lineno = lineno;
		 return CATCH;}
"float"		{write(fd, yytext, yyleng);return FLOAT;}
"long"		{write(fd, yytext, yyleng);return LONG;}
"short"		{write(fd, yytext, yyleng);return SHORT;}
"volatile"	{write(fd, yytext, yyleng);return VOLATILE;}
"char"		{write(fd, yytext, yyleng);return CHAR;}
"for"		{write(fd, yytext, yyleng);
		 yylval.s_str1 = allocate_one_element(struct_def1);
		 check_mem_alloc_error(yylval.s_str1);
		 yylval.s_str1->lineno = lineno;
		 return FOR;}
"native"	{write(fd, yytext, yyleng);return NATIVE;}
"static"	{write(fd, yytext, yyleng);return STATIC;}
"while"		{write(fd, yytext, yyleng);
		 yylval.s_str1 = allocate_one_element(struct_def1);
		 check_mem_alloc_error(yylval.s_str1);
		 yylval.s_str1->lineno = lineno;
		 return WHILE;}
"class"		{write(fd, yytext, yyleng);return CLASS;}
"future"	{write(fd, yytext, yyleng);return FUTURE;}
"new"		{write(fd, yytext, yyleng);return NEW;}
"super"		{write(fd, yytext, yyleng);return SUPER;}
"const"		{write(fd, yytext, yyleng);return CONST;}
"generic"	{write(fd, yytext, yyleng);return GENERIC;}
"null"		{write(fd, yytext, yyleng);return JNULL;}
"switch"	{write(fd, yytext, yyleng);return SWITCH;}
"continue"	{write(fd, yytext, yyleng);
		 before_control_flow_statements(8);
	 	 return CONTINUE;}
"goto"		{write(fd, yytext, yyleng);return GOTO;}
"operator"	{write(fd, yytext, yyleng);return OPERATOR;}
"synchronized"	{write(fd, yytext, yyleng);return SYNCHRONIZED;}
"default"	{before_control_flow_statements(0);
                 write(fd, yytext, yyleng);
		 yylval.s_str1 = allocate_one_element(struct_def1);
		 check_mem_alloc_error(yylval.s_str1);
		 yylval.s_str1->lineno = lineno;
		 return DEFAULT;}
"if"		{write(fd, yytext, yyleng);
		 yylval.s_str1 = allocate_one_element(struct_def1);
		 check_mem_alloc_error(yylval.s_str1);
		 yylval.s_str1->lineno = lineno;
		 is_parsing_if = 1;return IF;}
"outer"		{write(fd, yytext, yyleng);return OUTER;}
"this"		{write(fd, yytext, yyleng);return THIS;}

{xExit}		{
		 before_control_flow_statements(0);
		 write(fd, "pack.Test.print_result();", 25);
		 write(fd, yytext, yyleng); 
		}

{Identifier}	{
		 yylval.s_str1 = allocate_one_element(struct_def1);
		 check_mem_alloc_error(yylval.s_str1);
		 yylval.s_str1->lb_offset = lseek(fd, 0, SEEK_CUR);
                 write(fd, yytext, yyleng);
		 yylval.s_str1->name = strdup(yytext);
		 check_mem_alloc_error(yylval.s_str1->name);
		 yylval.s_str1->lineno = lineno;
		 return IDENTIFIER;}

{DecimalNum}    {write(fd, yytext, yyleng);return LITERAL;}
{OctalNum}      {write(fd, yytext, yyleng);return LITERAL;}
{HexNum}        {write(fd, yytext, yyleng);return LITERAL;}

{Float1}        {write(fd, yytext, yyleng);return LITERAL;}
{Float2}        {write(fd, yytext, yyleng);return LITERAL;}
{Float3}        {write(fd, yytext, yyleng);return LITERAL;}
{Float4}        {write(fd, yytext, yyleng);return LITERAL;}

{Character}     {write(fd, yytext, yyleng);return LITERAL;}
{String}	{write(fd, yytext, yyleng);return LITERAL;}

{CR}   		{/*write(fd, yytext, yyleng); */}
{LF}		{statement_count_data *e;
		 e = allocate_one_element(statement_count_data);
		 check_mem_alloc_error(e);
		 e->lineno = lineno;
		 e->count = statement_counter;
		 if(is_inside_class == 1)
		 	e->count = 0;
		 write(fd, yytext, yyleng); 
		 #ifdef DEBUG
		 printf(" Line no = %d Count = %d e->count = %d \n",lineno,statement_counter, e->count);		
		 #endif
		 stat_count_list = insert_statement_count(stat_count_list, e);
		 lineno++;
		 statement_counter = 0;}
{FF}		{write(fd, yytext, yyleng);}
{TAB}		{write(fd, yytext, yyleng);}
{BLK}           {write(fd, yytext, yyleng);}
{BLANK}		{write(fd, yytext, yyleng);}

{Comment2}	{
		 #ifdef COMMENT_LINENO_COUNT
		 comment_lineno_count++;
		 #endif
		}
{Comment1}	{int i;
		  for( i = 0; i < yyleng - 3 ; i++)
		  	if(yytext[i+2] == '\n')   {
				statement_count_data *e;
				write(fd, "\n", 1);
		 		e = allocate_one_element(statement_count_data);
				e->lineno = lineno;				 						    e->count = statement_counter;
		 		e->count = 0;
		 		stat_count_list = insert_statement_count(stat_count_list, e);
		 		lineno++;
				#ifdef COMMENT_LINENO_COUNT
				comment_lineno_count++;
				#endif
				statement_counter = 0;
			}
		}
%%

void before_control_flow_statements(int index)   {
	block_element e;
	int start_index;
	if(linear_stmt_flag == ON)   {
		start_index = lseek(fd, 0, SEEK_CUR) - index;
		end_lineno = lineno;
		e = allocate_one_element(block_data);
		check_mem_alloc_error(e);
		e->start_index = start_index;
		e->start_lineno = start_lineno;
		e->end_lineno = end_lineno;
		block_list = insert_block(block_list, e);
		start_lineno = lineno;
		linear_stmt_flag = OFF;
	}
	start_lineno = lineno;
	return;
}

int skip_the_class(char *class_name)   {
	int i, found = 0, counter = 0;
	char ch;
	
	for(i = 0; i < number_of_classes; i++)  
		if(strcmp(classes_list[i], class_name) == 0)
			found = 1;
	if((found && classes_exclude_flag)||(!found && classes_include_flag))   {
		while(1)   {
			ch = input();
			write(fd, &ch, 1);
			if(counter == 0 && ch == '}')  {
				lseek(fd, -1, SEEK_CUR);
				unput(ch);
				break;
			}
			if( ch == '}')
				counter --;
			if( ch == '{')
				counter++;
			if( ch == '\n')   {
				statement_count_data *e;
		 		e = allocate_one_element(statement_count_data);
				e->lineno = lineno;
				e->count = statement_counter;
		 		stat_count_list = insert_statement_count(stat_count_list, e);
		 		lineno++;
				statement_counter = 0;
			}
		}
		return 0;
	}
	if((found && classes_include_flag)||(!found && classes_include_flag))  
		return 1;
	return 1; /* Added to avoid warning */
}

