/* Grammar specification file
 * Grammar is used from www.home.attbi.com/~bronnikov/java.html
 */
/**
 * @Developed by N V Sreenivasa Rao
 * @Supervisor   Prof Pankaj Jalote
 * @URL          http://www.cse.iitk.ac.in/users/jalote/download/javacoverage/index.html
 * @title        JavaCoverage
 * @version      1.0
 */
 
%{
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "extflags.h"
#include "block_list.h"
#include "func_class_list.h"
#include "branch_coverage_list.h"
#include "file_times_list.h"
#include "stack.h"
#include "statement_count_list.h"
#include "token.h"
#include "instrument.h"
int return_code = 0;
extern FILE *yyin;
extern int lineno;
extern char * yytext;
int fd;
linkedlist * class_list; 
linkedlist * func_list; 
linkedlist * v_t_class_list = NULL; 
linkedlist * v_t_func_list = NULL; 
linkedlist * v_t_branch_cov_list = NULL;
linkedlist * v_t_block_list = NULL; 
linkedlist * v_t_stat_count_list = NULL;
brace_stack_ptr v_b_stack;
extern void before_control_flow_statements(int); 
extern int skip_the_class(char *);
extern int yyleng; 
#define FILE_NAME "temp.jcov"
%}

%union   { 
	struct_def1 * s_str1;   /* This is for communication for lex and yacc */
} 

%start Goal

%token ABSTRACT
%token BOOLEAN BREAK BYTE BYVALUE
%token CASE CAST CATCH CHAR CLASS CONST CONTINUE
%token DEFAULT DO DOUBLE
%token ELSE EXTENDS
%token FINAL FINALLY FLOAT FOR FUTURE
%token GENERIC GOTO
%token IF IMPLEMENTS IMPORT INNER INSTANCEOF INT INTERFACE
%token LONG
%token NATIVE NEW JNULL
%token OPERATOR OUTER
%token PACKAGE PRIVATE PROTECTED PUBLIC
%token REST RETURN
%token SHORT STATIC SUPER SWITCH SYNCHRONIZED
%token THIS THROW THROWS TRANSIENT TRY
%token VAR VOID VOLATILE
%token WHILE
%token OP_INC OP_DEC
%token OP_SHL OP_SHR OP_SHRR
%token OP_GE OP_LE OP_EQ OP_NE
%token OP_LAND OP_LOR
%token OP_DIM
%token ASS_MUL ASS_DIV ASS_MOD ASS_ADD ASS_SUB
%token ASS_SHL ASS_SHR ASS_SHRR ASS_AND ASS_XOR ASS_OR
%token LITERAL BOOLLIT
%token <s_str1> IDENTIFIER IF FOR WHILE CASE DEFAULT CATCH FINALLY

%type <s_str1> ForExpr CatchHeader

%type <s_str1> DeclaratorName  MethodDeclarator ConstructorDeclarator

%type <s_str1> MethodBody Block Statement xElseStatement SelectionStatement 
		IterationStatement GuardingStatement JumpStatement 
		xtryblockfactoring
%type <s_str1> LeftParenthesis RightParenthesis


%%


Goal 
	: { unlink(FILE_NAME);
	    fd = open(FILE_NAME, O_RDWR | O_CREAT, 0666 );
	    if(fd == -1)   {
	    	fprintf(stderr, "Can not open or create file = %s \n", FILE_NAME);
		exit(1);
	    }
	  }
	  CompilationUnit
	;


TypeSpecifier
	: TypeName
	| TypeName Dims
	;

TypeName
	: PrimitiveType
	| QualifiedName
	;

ClassNameList
        : QualifiedName
        | ClassNameList ',' QualifiedName
	;

PrimitiveType
	: BOOLEAN
	| CHAR
	| BYTE
	| SHORT
	| INT
	| LONG
	| FLOAT
	| DOUBLE
	| VOID
	;

SemiColons
	: ';'
        | SemiColons ';'
        ;

CompilationUnit
	: ProgramFile
        ;

ProgramFile
	: PackageStatement ImportStatements TypeDeclarations
	| PackageStatement ImportStatements
	| PackageStatement                  TypeDeclarations
	|                  ImportStatements TypeDeclarations
	| PackageStatement
	|                  ImportStatements
	|                                   TypeDeclarations
	;

PackageStatement
	: PACKAGE QualifiedName SemiColons
		{index_after_import_statements = lseek(fd, 0, SEEK_CUR);
		 index_after_import_statements -= yyleng;
		}
	;

TypeDeclarations
	: TypeDeclarationOptSemi
	| TypeDeclarations TypeDeclarationOptSemi
	;

TypeDeclarationOptSemi
        : TypeDeclaration
        | TypeDeclaration SemiColons
        ;

ImportStatements
	: ImportStatement	
		{index_after_import_statements = lseek(fd, 0, SEEK_CUR);
		 index_after_import_statements -= yyleng;
		}
	| ImportStatements ImportStatement
		{index_after_import_statements = lseek(fd, 0, SEEK_CUR);
		 index_after_import_statements -= yyleng;
		}
	;

ImportStatement
	: IMPORT QualifiedName SemiColons
	| IMPORT QualifiedName '.' '*' SemiColons
	;

QualifiedName
	: IDENTIFIER	{ free($1); }
	| QualifiedName '.' IDENTIFIER { free($3);}
	;

TypeDeclaration
	: ClassHeader '{' FieldDeclarations '}'  
		{ class_list = rotate_left(class_list);}
	| ClassHeader '{' '}'                         
		{ class_list = rotate_left(class_list);}
	;

ClassHeader
	: Modifiers ClassWord IDENTIFIER Extends Interfaces 
		{if(skip_the_class($3->name))   {
		 	class_list = insert_class_list(class_list, $3->name, current_file_number);
		 	is_inside_class = 1; 
		 }
		}
	| Modifiers ClassWord IDENTIFIER Extends
		{if(skip_the_class($3->name))   {
		  	class_list = insert_class_list(class_list, $3->name, current_file_number);
		 	is_inside_class = 1; 
		 }
		}
	| Modifiers ClassWord IDENTIFIER       Interfaces
		{if(skip_the_class($3->name))   {
			class_list = insert_class_list(class_list, $3->name, current_file_number);
		 	is_inside_class = 1; 
		 }
		}
	|           ClassWord IDENTIFIER Extends Interfaces
		{if(skip_the_class($2->name))   {
			class_list = insert_class_list(class_list, $2->name, current_file_number);
		 	is_inside_class = 1; 
		 }
		}
	| Modifiers ClassWord IDENTIFIER
		{if(skip_the_class($3->name))   {
			class_list = insert_class_list(class_list, $3->name, current_file_number);
		 	is_inside_class = 1; 
		 }
		}
	|           ClassWord IDENTIFIER Extends
		{if(skip_the_class($2->name))   {
			class_list = insert_class_list(class_list, $2->name, current_file_number);
		 	is_inside_class = 1; 
		 }
		}
	|           ClassWord IDENTIFIER       Interfaces
		{if(skip_the_class($2->name))   {
			class_list = insert_class_list(class_list, $2->name, current_file_number);
		 	is_inside_class = 1; 
		 }
		}
	|           ClassWord IDENTIFIER
		{if(skip_the_class($2->name))   {
			class_list = insert_class_list(class_list, $2->name, current_file_number);
		 	is_inside_class = 1; 
		 }	
		}
	;

Modifiers
	: Modifier
	| Modifiers Modifier
	;

Modifier
	: ABSTRACT
	| FINAL
	| PUBLIC
	| PROTECTED
	| PRIVATE
	| STATIC
	| TRANSIENT
	| VOLATILE
	| NATIVE
	| SYNCHRONIZED
	;

ClassWord
	: CLASS
	| INTERFACE
	;

Interfaces
	: IMPLEMENTS ClassNameList
	;

FieldDeclarations
	: FieldDeclarationOptSemi
        | FieldDeclarations FieldDeclarationOptSemi
	;

FieldDeclarationOptSemi
        : FieldDeclaration
        | FieldDeclaration SemiColons
        ;

FieldDeclaration
	: FieldVariableDeclaration ';'
	| MethodDeclaration
	| ConstructorDeclaration
	| StaticInitializer
        | NonStaticInitializer
        | TypeDeclaration
	;

FieldVariableDeclaration
	: Modifiers TypeSpecifier VariableDeclarators
	|           TypeSpecifier VariableDeclarators
	;

VariableDeclarators
	: VariableDeclarator
	| VariableDeclarators ',' VariableDeclarator 
		{ statement_counter = (statement_counter <= 0)? 1: statement_counter-1;}
	;

VariableDeclarator
	: DeclaratorName
	| DeclaratorName '=' VariableInitializer    
		{ statement_counter++; linear_stmt_flag = ON;} 
	;

VariableInitializer
	: Expression
	| '{' '}'
        | '{' ArrayInitializers '}'
        ;

ArrayInitializers
	: VariableInitializer
	| ArrayInitializers ',' VariableInitializer
	| ArrayInitializers ','
	;

MethodDeclaration
	: Modifiers TypeSpecifier MethodDeclarator Throws MethodBody
		{	if($5->tok == block)   {
				before_control_flow_statements(1); 
				func_list = insert_method_list (func_list, 
					get_element(class_data *, class_list), 
					$3->name, $5->start_lineno, 
					$5->end_lineno, $5->lb_offset, 
					$5->rb_offset); 
			}
			is_inside_class = 1;
		}
	| Modifiers TypeSpecifier MethodDeclarator        MethodBody
		{	if($4->tok == block)   {
				before_control_flow_statements(1); 
				func_list = insert_method_list (func_list, 
				get_element(class_data *, class_list), 
				$3->name, $4->start_lineno, $4->end_lineno, 
				$4->lb_offset, $4->rb_offset); 
			}
			is_inside_class = 1;
		}
	|           TypeSpecifier MethodDeclarator Throws MethodBody
		{	if($4->tok == block)   {
				before_control_flow_statements(1); 
				func_list = insert_method_list (func_list,  
				get_element(class_data *, class_list), 
				$2->name, $4->start_lineno, $4->end_lineno, 
				$4->lb_offset, $4->rb_offset); 
			}
			is_inside_class = 1;
		}
	|           TypeSpecifier MethodDeclarator        MethodBody
		{	if($3->tok == block)   {
				before_control_flow_statements(1); 
				func_list = insert_method_list (func_list, 
				get_element(class_data *, class_list), 
				$2->name, $3->start_lineno, $3->end_lineno, 
				$3->lb_offset, $3->rb_offset); 
			}
			is_inside_class = 1;
		}
	;

MethodDeclarator
	: DeclaratorName '(' ParameterList ')'  { $$ = $1; is_inside_class = 0;
						  #ifdef DEBUG
						  printf("\t\tDECLARATOR NAME = %s \n", $1->name);
						  #endif
						  linear_stmt_flag = OFF;
						  start_lineno = lineno;
						  is_method_block = 1;}
	| DeclaratorName '(' ')'                { $$ = $1; is_inside_class = 0;
						  #ifdef DEBUG
						  printf("\t\tDECLARATOR NAME = %s address = %x \n", $1->name, $1->name);
						  #endif
						  linear_stmt_flag = OFF;
						  start_lineno = lineno;
						  is_method_block = 1;}
	| MethodDeclarator OP_DIM               { $$ = $1; is_inside_class = 0;
						  linear_stmt_flag = OFF;
						  start_lineno = lineno;
						  is_method_block = 1;}
	;

ParameterList
	: Parameter
	| ParameterList ',' Parameter
	;

Parameter
	: TypeSpecifier DeclaratorName
        | FINAL TypeSpecifier DeclaratorName
	;

DeclaratorName
	: IDENTIFIER                    { $$ = $1; }
        | DeclaratorName OP_DIM         { $$ = $1; }
        ;

Throws
	: THROWS ClassNameList
	;

MethodBody
	: Block   { $$ = $1; $$->tok = block; }
	| ';'	  { struct_def1 *temp;
		    temp = allocate_one_element(struct_def1);
		    check_mem_alloc_error(temp);
		    $$ = temp;
		    $$->tok = not_block; }
	;

ConstructorDeclaration
	: Modifiers ConstructorDeclarator Throws Block
		{ func_list = insert_method_list (func_list,  
			get_element(class_data *, class_list), 
			$2->name, $4->start_lineno, $4->end_lineno, 
			give_after_this_super(fd, $4->lb_offset), $4->rb_offset); 
		  before_control_flow_statements(1); 
		}
		
	| Modifiers ConstructorDeclarator        Block
		{ func_list = insert_method_list (func_list, 
			get_element(class_data *, class_list), 	
			$2->name, $3->start_lineno, $3->end_lineno, 
			give_after_this_super(fd, $3->lb_offset),  $3->rb_offset); 
		  before_control_flow_statements(1); 
		}
		
	|           ConstructorDeclarator Throws Block
		{ func_list = insert_method_list (func_list, 
			get_element(class_data *, class_list), 
			$1->name, $3->start_lineno, $3->end_lineno, 
			give_after_this_super(fd, $3->lb_offset), $3->rb_offset); 
		  before_control_flow_statements(1); 
		 }
				
	|           ConstructorDeclarator        Block
		{ func_list = insert_method_list (func_list, 
			get_element(class_data *, class_list), 
			$1->name,  $2->start_lineno, $2->end_lineno, 
			give_after_this_super(fd, $2->lb_offset), $2->rb_offset); 
		  before_control_flow_statements(1); 
		}
	;

ConstructorDeclarator
	: IDENTIFIER '(' ParameterList ')' 
		{ $$ = $1; start_lineno = lineno;
		  is_inside_class = 0;
		}
	| IDENTIFIER '(' ')'		     
		{ $$ = $1; start_lineno = lineno;
		  is_inside_class = 0;
		}
	;

StaticInitializer
	: STATIC Block
	;

NonStaticInitializer
        : Block
        ;

Extends
	: EXTENDS TypeName
	| Extends ',' TypeName
	;

Block
	: LeftParenthesis LocalVariableDeclarationsAndStatements RightParenthesis   		
	        { $$ = $1; $$->rb_offset = $3->rb_offset;
		  $$->end_lineno = $3->end_lineno;}
	| LeftParenthesis RightParenthesis 
		{ $$ = $1; $$->rb_offset = $2->rb_offset;
		  $$->end_lineno = $2->end_lineno;}
        ;

LeftParenthesis
	: '{'   {struct_def1 *temp;
		 temp = allocate_one_element(struct_def1);
		 check_mem_alloc_error(temp);
		 temp->lb_offset = lseek(fd, 0, SEEK_CUR);
		 temp->start_lineno = lineno;
		 $$ = temp;
		 }
	;

RightParenthesis
	: '}'   { struct_def1 *temp;
	          temp = allocate_one_element(struct_def1);
		  check_mem_alloc_error(temp);
		  temp->rb_offset = lseek(fd, 0, SEEK_CUR);
		  temp->end_lineno = lineno;
		  $$ = temp;
		  }
	;

LocalVariableDeclarationsAndStatements
	: LocalVariableDeclarationOrStatement
	| LocalVariableDeclarationsAndStatements LocalVariableDeclarationOrStatement
	;

LocalVariableDeclarationOrStatement
	: LocalVariableDeclarationStatement
	| Statement
	;

LocalVariableDeclarationStatement
	: TypeSpecifier VariableDeclarators ';'
        | FINAL TypeSpecifier VariableDeclarators ';'
	;

Statement
	: EmptyStatement               
		{ $$ = allocate_one_element(struct_def1);
		  check_mem_alloc_error($$);
		  $$->start_lineno = $$->end_lineno = lineno;
		  $$->tok = not_block;
		}
	| LabelStatement               { $$ = NULL; }
	| ExpressionStatement ';'                    
		{ statement_counter++;
		  linear_stmt_flag = ON;
		  $$ = allocate_one_element(struct_def1);
		  check_mem_alloc_error($$);
		  $$->start_lineno = $$->end_lineno = lineno;
		  $$->tok = not_block;
		}
        | SelectionStatement		{ $$ = allocate_one_element(struct_def1);
				          check_mem_alloc_error($$);
					  $$->tok = not_block; }
        | IterationStatement		{ $$ = allocate_one_element(struct_def1);
					  check_mem_alloc_error($$);
					  $$->tok = not_block; }
	| JumpStatement			
					{ $$ = allocate_one_element(struct_def1);
					  check_mem_alloc_error($$);
					  $$->tok = not_block; }
	| GuardingStatement		{ $$ = allocate_one_element(struct_def1);
					  check_mem_alloc_error($$);
					  $$->tok = not_block;} 
	| Block				{ $$ = $1; }
	;

EmptyStatement
	: ';'
        ;

LabelStatement
	: IDENTIFIER {/*squeeze out the space*/ before_control_flow_statements(lseek(fd, 0, SEEK_CUR) - $1->lb_offset);}
	  xColon 
		{ free($1);}
        | CASE ConstantExpression ':' 
		{int first_index; 
		 linear_stmt_flag = OFF; start_lineno = lineno; 
		 first_index = lseek(fd, 0, SEEK_CUR) ;
		 branch_cov_list = insert_branch_coverage_list(branch_cov_list, $1->lineno, first_index, BRANCH);
		 free($1); 		 
		 }
	| DEFAULT ':' 
		{int first_index; 
		 linear_stmt_flag = OFF; 
		 first_index = lseek(fd, 0, SEEK_CUR);
		 branch_cov_list = insert_branch_coverage_list(branch_cov_list, $1->lineno, first_index, BRANCH);
		 free($1); 		 
		}
        ;

ExpressionStatement
	: Expression
	;

SelectionStatement
	: IF xbeforecontrolflow '(' Expression ')' xsinglestatement
	  Statement   
		{ 	
			block_element e1;
			if($7->tok == block)  {
				end_lineno = $7->end_lineno;
				start_index = $7->rb_offset;
				start_index--;
				brace_stack_pop(v_b_stack);
			}
			else   {
				int offset = 0;
				int temp;
				simple_statements_start_lineno = give_top_start_lineno(v_b_stack);
				temp = brace_stack_pop(v_b_stack);
				offset = lseek(fd, 0, SEEK_CUR);
				lseek(fd, temp, SEEK_SET);
				write(fd, "{", 1);
				$7->lb_offset = lseek(fd, 0, SEEK_CUR);
				if(strcmp(yytext, ";") != 0)  {
					lseek(fd, offset - yyleng, SEEK_SET);
					write(fd, "}", 1);
					$7->rb_offset = lseek(fd, 0, SEEK_CUR);
					start_index = offset - yyleng;
					write(fd, yytext, yyleng);
					end_lineno = lineno;
					start_lineno = simple_statements_start_lineno;
				}
				else  {
					lseek(fd, offset, SEEK_SET);
					write(fd, "}", 1);
					$7->rb_offset = lseek(fd, 0, SEEK_CUR);
					start_index = lseek(fd, 0, SEEK_CUR);
					start_index--;
					end_lineno = lineno;	
					start_lineno = end_lineno ;
				}
			}
			e1 = allocate_one_element(block_data);
			check_mem_alloc_error(e1);
			e1->start_lineno = start_lineno;
			e1->end_lineno = end_lineno;
			e1->start_index = start_index;
			start_lineno = end_lineno + 1;
			if(linear_stmt_flag == ON)   
				block_list = insert_block(block_list, e1);
			linear_stmt_flag = OFF;
		}
	  xElseStatement
	  	{
			block_element e1;
			if($9->lineno != -1)   { /*lineno = -1 for added else*/
			if($9->tok == block)  {
				end_lineno = $9->end_lineno;
				start_index = $9->rb_offset;
				start_index--;
				brace_stack_pop(v_b_stack);
			}
			else   {
				int offset = 0;
				int temp;
				simple_statements_start_lineno = give_top_start_lineno(v_b_stack);
				temp = brace_stack_pop(v_b_stack);
				offset = lseek(fd, 0, SEEK_CUR);
				lseek(fd, temp, SEEK_SET);
				write(fd, "{", 1);
				$9->lb_offset = lseek(fd, 0, SEEK_CUR);
				if(strcmp(yytext, ";") != 0)  {
					lseek(fd, offset - yyleng, SEEK_SET);
					write(fd, "}", 1);
					$9->rb_offset = lseek(fd, 0, SEEK_CUR);
					start_index = offset - yyleng;
					write(fd, yytext, yyleng);
					end_lineno = lineno;
					start_lineno = simple_statements_start_lineno;
				}   else  {
					lseek(fd, offset, SEEK_SET);
					write(fd, "}", 1);
					$9->rb_offset = lseek(fd, 0, SEEK_CUR);
					start_index = lseek(fd, 0, SEEK_CUR);
					start_index--;
					end_lineno = lineno;	
					start_lineno = end_lineno ;
				}
			}
			e1 = allocate_one_element(block_data);
			e1->start_lineno = start_lineno;
			e1->end_lineno = end_lineno;
			e1->start_index = start_index;
			start_lineno = end_lineno + 1;
			if(linear_stmt_flag == ON)
				block_list = insert_block(block_list, e1);
			linear_stmt_flag = OFF;
			}
		}
		{
			int lineno, first_index, second_index;
			lineno = $1->lineno;
			first_index = $7->lb_offset;
			second_index = $9->lb_offset;
			branch_cov_list = insert_branch_coverage_list(branch_cov_list, lineno, first_index, TRUE);
			branch_cov_list = insert_branch_coverage_list(branch_cov_list, lineno, second_index, FALSE);
			free($1); /*free($7); free($9);*/
		}

        | SWITCH { before_control_flow_statements(6); }
	  '(' Expression ')' Block
	  	{$$ = $6; }
        ;

xElseStatement 
	: /* empty */  { 
		$$ = allocate_one_element(struct_def1); 
		check_mem_alloc_error($$);
		$$->lineno = -1; /* This is to tell that it is the added else*/
		#ifdef DEBUG
		printf("Now i am trying to add else  yytext = %s \n", yytext);
		#endif
		lseek(fd, -yyleng, SEEK_CUR);  
		write(fd, "else ;", 6);
		$$->lb_offset = $$->rb_offset = lseek(fd, 0, SEEK_CUR) - 1;
		write(fd, yytext, yyleng);
		}
        | ELSE 		
	       { brace_stack_push(v_b_stack, lseek(fd, 0, SEEK_CUR)/*+1*/, lineno);
	       }
	  Statement
		{ 	$$ = $3;
		}
	;
	
IterationStatement
	: WHILE { before_control_flow_statements(5);
		}
	  '(' Expression ')' xsinglestatement
	  Statement
	  	{
			block_element e1;
			int t_lineno, t_first_index, t_second_index;
			if($7->tok == block)  {
				end_lineno = $7->end_lineno;
				start_index = $7->rb_offset;
				start_index--;
				brace_stack_pop(v_b_stack);
				t_first_index = $7->lb_offset;
				t_second_index = $7->rb_offset;
			}
			else   {
				int offset = 0;
				int temp;
				simple_statements_start_lineno = give_top_start_lineno(v_b_stack);
				temp = brace_stack_pop(v_b_stack);
				offset = lseek(fd, 0, SEEK_CUR);
				lseek(fd, temp, SEEK_SET);
				write(fd, "{", 1);
				t_first_index = lseek(fd, 0, SEEK_CUR);
				if(strcmp(yytext, ";") != 0)  {
					lseek(fd, offset - yyleng, SEEK_SET);
					write(fd, "}", 1);
				t_second_index = lseek(fd, 0, SEEK_CUR);
					start_index = offset - yyleng;
					write(fd, yytext, yyleng);
					end_lineno = lineno;
					start_lineno = simple_statements_start_lineno;
				}
				else  {
					lseek(fd, offset, SEEK_SET);
					write(fd, "}", 1);
				t_second_index = lseek(fd, 0, SEEK_CUR);
					start_index = lseek(fd, 0, SEEK_CUR);
					start_index--;
					end_lineno = lineno;	
					start_lineno = end_lineno + 1;
				}
			}
			e1 = allocate_one_element(block_data);
			check_mem_alloc_error(e1);
			e1->start_lineno = start_lineno;
			e1->end_lineno = end_lineno;
			e1->start_index = start_index;
			start_lineno = end_lineno + 1;
			if(linear_stmt_flag == ON)
				block_list = insert_block(block_list, e1);
			linear_stmt_flag = OFF;
			t_lineno = $1->lineno;
			branch_cov_list = insert_branch_coverage_list(branch_cov_list, t_lineno, t_first_index, TRUE);
			branch_cov_list = insert_branch_coverage_list(branch_cov_list, t_lineno, t_second_index, FALSE);
			free($1); 
		}
	| DO xsinglestatement
	  Statement
	  	{
			block_element e1;
			if($3->tok == block)  {
				end_lineno = $3->end_lineno;
				start_index = $3->rb_offset;
				start_index--;
				brace_stack_pop(v_b_stack);
			}
			else   {
				int offset = 0;
				int temp;
				simple_statements_start_lineno = give_top_start_lineno(v_b_stack);
				temp = brace_stack_pop(v_b_stack);
				offset = lseek(fd, 0, SEEK_CUR);
				lseek(fd, temp, SEEK_SET);
				write(fd, "{", 1);
				$3->lb_offset= lseek(fd, 0, SEEK_CUR);
				if(strcmp(yytext, ";") != 0)  {
					lseek(fd, offset - yyleng, SEEK_SET);
					write(fd, "}", 1);
					$3->rb_offset = offset - yyleng;
					write(fd, yytext, yyleng);
					end_lineno = lineno;
					start_lineno = simple_statements_start_lineno;
				}
				else  {
					lseek(fd, offset, SEEK_SET);
					write(fd, "}", 1);
					$3->rb_offset = lseek(fd, 0, SEEK_CUR);
					start_index--;
					end_lineno = lineno;	
					start_lineno = end_lineno + 1;
				}
			}
			e1 = allocate_one_element(block_data);
			check_mem_alloc_error(e1);
			e1->start_lineno = start_lineno;
			e1->end_lineno = end_lineno;
			e1->start_index = start_index;
			start_lineno = end_lineno + 1;
			if(linear_stmt_flag == ON)
				block_list = insert_block(block_list, e1);
			linear_stmt_flag = OFF;
		}
	  
	  WHILE 
	  '('   {
	  	int offset = lseek(fd, -yyleng, SEEK_CUR);
		write(fd, "(,", 2);
		write(fd, yytext, yyleng);
		branch_cov_list = insert_branch_coverage_list(branch_cov_list, $5->lineno, offset + 1, DO_WHILE_TRUE); 
	  }
	  
	  Expression 
	  ')' {
	  	lseek(fd, -yyleng, SEEK_CUR);
		write(fd, "))", 2);
		write(fd, yytext, yyleng);
	  }
	  
	  ';' 
	  {
	  	int t_lineno, t_second_index;
		t_lineno = $5->lineno;
		t_second_index = lseek(fd, 0, SEEK_CUR);
		branch_cov_list = insert_branch_coverage_list(branch_cov_list, t_lineno, t_second_index, FALSE);
		free($5); 
	  }
	
	| FOR xbeforecontrolflow_FOR
	  '(' ForInit ForExpr ForIncr ')' xsinglestatement 
	  Statement
	  {
		block_element e1;
		int t_lineno, t_first_index, t_second_index;
		if($9->tok == block)  {
			end_lineno = $9->end_lineno;
			start_index = $9->rb_offset;
			start_index--;
			brace_stack_pop(v_b_stack);
			t_first_index = $9->lb_offset;
			t_second_index = $9->rb_offset;
		}
		else   {
			int offset = 0;
			int temp;
			simple_statements_start_lineno = give_top_start_lineno(v_b_stack);
			temp = brace_stack_pop(v_b_stack);
			offset = lseek(fd, 0, SEEK_CUR);
			lseek(fd, temp, SEEK_SET);
			write(fd, "{", 1);
			t_first_index = lseek(fd, 0, SEEK_CUR);
			
			if(strcmp(yytext, ";") != 0)  {
				lseek(fd, offset - yyleng, SEEK_SET);
				write(fd, "}", 1);
				t_second_index = lseek(fd, 0, SEEK_CUR);
				start_index = offset - yyleng;
				write(fd, yytext, yyleng);
				end_lineno = lineno;
				start_lineno = simple_statements_start_lineno;
			}
			else  {
				lseek(fd, offset, SEEK_SET);
				write(fd, "}", 1);
				t_second_index = lseek(fd, 0, SEEK_CUR);
				start_index = lseek(fd, 0, SEEK_CUR);
				start_index--;
				end_lineno = lineno;	
				start_lineno = end_lineno + 1;
			}
		}
		e1 = allocate_one_element(block_data);
		check_mem_alloc_error(e1);
		#ifdef DEBUG
		printf("Now it is in FOR start_lineno = %d \n", start_lineno);
		#endif
		e1->start_lineno = start_lineno;
		e1->end_lineno = end_lineno;
		e1->start_index = start_index;
		start_lineno = end_lineno + 1;					
		if(linear_stmt_flag == ON)   
			block_list = insert_block(block_list, e1);
		linear_stmt_flag = OFF;
			
		t_lineno = $1->lineno;
		#ifdef DEBUG
		print_branch_coverage_list(branch_cov_list);
		#endif
		branch_cov_list = insert_branch_coverage_list(branch_cov_list, t_lineno, t_first_index, TRUE);
		if($5 == NULL)   
		branch_cov_list = insert_branch_coverage_list(branch_cov_list, t_lineno, t_second_index, FALSE);
		else
			free($5);
		free($1); /*free($9);*/
	}
	
	
	| FOR xbeforecontrolflow_FOR
	  '(' ForInit ForExpr         ')' xsinglestatement
	Statement
	  {
		block_element e1;
		int t_lineno, t_first_index, t_second_index;
		if($8->tok == block)  {
			end_lineno = $8->end_lineno;
			start_index = $8->rb_offset;
			start_index--;
			brace_stack_pop(v_b_stack);
			t_first_index = $8->lb_offset;
			t_second_index = $8->rb_offset;
		}
		else   {
			int offset = 0;
			int temp;
			simple_statements_start_lineno = give_top_start_lineno(v_b_stack);
			temp = brace_stack_pop(v_b_stack);
			offset = lseek(fd, 0, SEEK_CUR);
			lseek(fd, temp, SEEK_SET);
			write(fd, "{", 1);
			t_first_index = lseek(fd, 0, SEEK_CUR);
			if(strcmp(yytext, ";") != 0)  {
				lseek(fd, offset - yyleng, SEEK_SET);
				write(fd, "}", 1);
				t_second_index = lseek(fd, 0, SEEK_CUR);
				start_index = offset - yyleng;
				write(fd, yytext, yyleng);
				end_lineno = lineno;
				start_lineno = simple_statements_start_lineno;
			}
			else  {
				lseek(fd, offset, SEEK_SET);
				write(fd, "}", 1);
				t_second_index = lseek(fd, 0, SEEK_CUR);
				start_index = lseek(fd, 0, SEEK_CUR);
				start_index--;
				end_lineno = lineno;	
				start_lineno = end_lineno + 1;
			}
		}
		e1 = allocate_one_element(block_data);
		check_mem_alloc_error(e1);
		e1->start_lineno = start_lineno;
		e1->end_lineno = end_lineno;
		e1->start_index = start_index;
		start_lineno = end_lineno + 1;
		if(linear_stmt_flag == ON)
			block_list = insert_block(block_list, e1);
		linear_stmt_flag = OFF;
		t_lineno = $1->lineno;
		branch_cov_list = insert_branch_coverage_list(branch_cov_list, t_lineno, t_first_index, TRUE);
		if($5 == NULL)   
		branch_cov_list = insert_branch_coverage_list(branch_cov_list, t_lineno, t_second_index, FALSE);
		else
			free($5);
		free($1); /*free($8);*/
	}
	;

ForInit
	: ExpressionStatements ';'
	| LocalVariableDeclarationStatement
	| ';'
	;

ForExpr
	: Expression ';'  { $$ = NULL;}
	| ';'  { $$ = allocate_one_element(struct_def1);
		 check_mem_alloc_error($$);
	       }
	;

ForIncr
	: ExpressionStatements
	;

ExpressionStatements
	: ExpressionStatement
	| ExpressionStatements ',' ExpressionStatement
	;

JumpStatement
	: BREAK   
	  IDENTIFIER ';'	
		{ statement_counter++;
		  free($2);
		}
	| BREAK            
	   ';'  
		{ statement_counter++;
		}
        | CONTINUE  { before_control_flow_statements(8); start_lineno++; }
	  IDENTIFIER ';' 
		{ statement_counter++;
		  free($3);
		}
	| CONTINUE  { before_control_flow_statements(8); start_lineno++; }          
	  ';' 
		{ statement_counter++;
		}
	| RETURN  
	  Expression ';'  
		{ statement_counter++;
		}
	| RETURN  { before_control_flow_statements(7); start_lineno++; }
	  ';'  
		{ start_lineno = lineno;
		  statement_counter++; 
		}
	| THROW  { before_control_flow_statements(5); start_lineno++; }
	  Expression ';'   
		{ statement_counter++;
		}
	;

GuardingStatement
	: SYNCHRONIZED '(' Expression ')' Statement   { $$ = NULL; }
	| TRY Block 
		{ 	
			block_element e1;
			end_lineno = $2->end_lineno;
			start_index = $2->rb_offset;
			start_index--;
			e1 = allocate_one_element(block_data);
			check_mem_alloc_error(e1);
			e1->start_lineno = start_lineno;
			e1->end_lineno = end_lineno;
			e1->start_index = start_index;
			start_lineno = end_lineno + 1;
			if(linear_stmt_flag == ON)   
				block_list = insert_block(block_list, e1);
			linear_stmt_flag = OFF;
		}
	  xtryblockfactoring		      { $$ = NULL; }		
	;

xtryblockfactoring
	: Finally		{ $$ = NULL; }
	| Catches		{ $$ = NULL; }
	| Catches Finally	{ $$ = NULL; }
	;

Catches
	: Catch		
	| Catches Catch
	;

Catch
	: CatchHeader Block
		{ 	
			block_element e1;
			end_lineno = $2->end_lineno;
			start_index = $2->rb_offset;
			start_index--;
			e1 = allocate_one_element(block_data);
			check_mem_alloc_error(e1);
			e1->start_lineno = start_lineno;
			e1->end_lineno = end_lineno;
			e1->start_index = start_index;
			start_lineno = end_lineno + 1;
			if(linear_stmt_flag == ON)   
				block_list = insert_block(block_list, e1);
			linear_stmt_flag = OFF;
			branch_cov_list = insert_branch_coverage_list(branch_cov_list, $1->lineno, $2->lb_offset, CATCH_FIN_EX);
			free($1);
		}
	;

CatchHeader
	: CATCH xcatchfactoring		
		{ start_lineno = lineno; 
		  $$ = $1;
		}
	;

xcatchfactoring
	: '(' TypeSpecifier IDENTIFIER ')' { free($3); }
	| '(' TypeSpecifier ')'
	;
Finally
	: FINALLY Block
		{ 	
			block_element e1;
			end_lineno = $2->end_lineno;
			start_index = $2->rb_offset;
			start_index--;
			e1 = allocate_one_element(block_data);
			check_mem_alloc_error(e1);
			e1->start_lineno = start_lineno;
			e1->end_lineno = end_lineno;
			e1->start_index = start_index;
			start_lineno = end_lineno + 1;
			if(linear_stmt_flag == ON)   
				block_list = insert_block(block_list, e1);
			linear_stmt_flag = OFF;
			branch_cov_list = insert_branch_coverage_list(branch_cov_list, $1->lineno, $2->lb_offset, CATCH_FIN_EX);
			free($1);
		}
	;

PrimaryExpression
	: QualifiedName
	| NotJustName
	;

NotJustName
	: SpecialName
	| NewAllocationExpression
	| ComplexPrimary
	;

ComplexPrimary
	: '(' Expression ')'
	| ComplexPrimaryNoParenthesis
	;

ComplexPrimaryNoParenthesis
	: LITERAL
	| BOOLLIT
	| ArrayAccess
	| FieldAccess
	| MethodCall
	;

ArrayAccess
	: QualifiedName '[' Expression ']'
	| ComplexPrimary '[' Expression ']'
	;

FieldAccess
	: NotJustName '.' IDENTIFIER   {free($3);}
	| RealPostfixExpression '.' IDENTIFIER  { free($3); }
        | QualifiedName '.' THIS
        | QualifiedName '.' CLASS
        | PrimitiveType '.' CLASS
	;

MethodCall
	: MethodAccess '(' ArgumentList ')'   
	| MethodAccess '(' ')' 		     
	;

MethodAccess
	: ComplexPrimaryNoParenthesis
	| SpecialName
	| QualifiedName
	;

SpecialName
	: THIS
	| SUPER
	| JNULL
	;

ArgumentList
	: Expression
	| ArgumentList ',' Expression
	;

NewAllocationExpression
        : PlainNewAllocationExpression
        | QualifiedName '.' PlainNewAllocationExpression
        ;

PlainNewAllocationExpression
    	: ArrayAllocationExpression
    	| ClassAllocationExpression
    	| ArrayAllocationExpression '{' '}'
    	| ClassAllocationExpression '{' '}'
    	| ArrayAllocationExpression '{' ArrayInitializers '}'
    	| ClassAllocationExpression '{' FieldDeclarations '}'
    	;

ClassAllocationExpression
	: NEW TypeName '(' ArgumentList ')'
	| NEW TypeName '('              ')'
        ;

ArrayAllocationExpression
	: NEW TypeName DimExprs Dims
	| NEW TypeName DimExprs
        | NEW TypeName Dims
	;

DimExprs
	: DimExpr
	| DimExprs DimExpr
	;

DimExpr
	: '[' Expression ']'
	;

Dims
	: OP_DIM
	| Dims OP_DIM
	;

PostfixExpression
	: PrimaryExpression
	| RealPostfixExpression
	;

RealPostfixExpression
	: PostfixExpression OP_INC
	| PostfixExpression OP_DEC
	;

UnaryExpression
	: OP_INC UnaryExpression
	| OP_DEC UnaryExpression
	| ArithmeticUnaryOperator CastExpression
	| LogicalUnaryExpression
	;

LogicalUnaryExpression
	: PostfixExpression
	| LogicalUnaryOperator UnaryExpression
	;

LogicalUnaryOperator
	: '~'
	| '!'
	;

ArithmeticUnaryOperator
	: '+'
	| '-'
	;

CastExpression
	: UnaryExpression
	| '(' PrimitiveTypeExpression ')' CastExpression
	| '(' ClassTypeExpression ')' CastExpression
	| '(' Expression ')' LogicalUnaryExpression
	;

PrimitiveTypeExpression
	: PrimitiveType
        | PrimitiveType Dims
        ;

ClassTypeExpression
	: QualifiedName Dims
        ;

MultiplicativeExpression
	: CastExpression
	| MultiplicativeExpression '*' CastExpression
	| MultiplicativeExpression '/' CastExpression
	| MultiplicativeExpression '%' CastExpression
	;

AdditiveExpression
	: MultiplicativeExpression
        | AdditiveExpression '+' MultiplicativeExpression
	| AdditiveExpression '-' MultiplicativeExpression
        ;

ShiftExpression
	: AdditiveExpression
        | ShiftExpression OP_SHL AdditiveExpression
        | ShiftExpression OP_SHR AdditiveExpression
        | ShiftExpression OP_SHRR AdditiveExpression
	;

RelationalExpression
	: ShiftExpression
        | RelationalExpression '<' ShiftExpression
	| RelationalExpression '>' ShiftExpression
	| RelationalExpression OP_LE ShiftExpression
	| RelationalExpression OP_GE ShiftExpression
	| RelationalExpression INSTANCEOF TypeSpecifier
	;

EqualityExpression
	: RelationalExpression
        | EqualityExpression OP_EQ RelationalExpression
        | EqualityExpression OP_NE RelationalExpression
        ;

AndExpression
	: EqualityExpression
        | AndExpression '&' EqualityExpression
        ;

ExclusiveOrExpression
	: AndExpression
	| ExclusiveOrExpression '^' AndExpression
	;

InclusiveOrExpression
	: ExclusiveOrExpression
	| InclusiveOrExpression '|' ExclusiveOrExpression
	;

ConditionalAndExpression
	: InclusiveOrExpression
	| ConditionalAndExpression OP_LAND InclusiveOrExpression
	;

ConditionalOrExpression
	: ConditionalAndExpression
	| ConditionalOrExpression OP_LOR ConditionalAndExpression
	;

ConditionalExpression
	: ConditionalOrExpression
	| ConditionalOrExpression '?' Expression ':' ConditionalExpression
	;

AssignmentExpression
	: ConditionalExpression
	| UnaryExpression AssignmentOperator AssignmentExpression
	;

AssignmentOperator
	: '='
	| ASS_MUL
	| ASS_DIV
	| ASS_MOD
	| ASS_ADD
	| ASS_SUB
	| ASS_SHL
	| ASS_SHR
	| ASS_SHRR
	| ASS_AND
	| ASS_XOR
	| ASS_OR
	;

Expression
	: AssignmentExpression
        ;

ConstantExpression
	: ConditionalExpression
	;

xbeforecontrolflow 
	: { before_control_flow_statements(2); }
	;

xbeforecontrolflow_FOR
	: { before_control_flow_statements(3);}
	;

xsinglestatement
	: /* empty */
	 {
	   brace_stack_push(v_b_stack, lseek(fd, 0, SEEK_CUR), lineno);
	   write(fd, " ", 1);
	  }
	;

xColon
	: ':'
	;

%%

void yyerror(char *string)   {
	fprintf(stderr, "\nParse Error in line %d in file = %s \n", lineno, output_file);
	return_code = lineno;
	yyin = NULL;
	exit(1);
	return;
}

void set_file_name(char *s)   {
	int len, i =0;
	int index = 0;
	len = strlen(s);
	while(s[i] != '\0')   {
		if(s[i] == FILE_SEPARATOR)   
			index = i+1;
		i++;
	}
	output_file = (char *)realloc(output_file, sizeof(char)*(len - index + 1));
	check_mem_alloc_error(output_file);
	i = 0;
	while(s[index] != '\0')   {
		output_file[i++] = s[index++];	
	}
	output_file[i] = '\0';
	return;
}

void make_hierarchy(char *s)   {
	char * dir_name;
	int index = 0;
	dir_name = (char *)strdup(s);
	check_mem_alloc_error(dir_name);
	while(s[index] != '\0')   {
		if(s[index] == '.')   {
			dir_name[index] = '\0';
			#ifdef DEBUG
			printf("The directory created = %s \n", dir_name);
			#endif
			mkdir(dir_name, 0777);
			s[index] = dir_name[index] = '/';
			index++;
		}
		dir_name[index] = s[index++];
	}
	mkdir(dir_name, 0777);
	return;
}

int set_file_name1(char *s)   {
	static int state = 0;
	static char package[1024] = "";
	static char *temp = "";
	
	switch(state)   {
	case 0: if(s[0] == '$')  { 
			state = 1;
			return 0;
		}
		else
			set_file_name(s);
		break;
	case 1: state = 2;
		make_hierarchy(s);
		strcpy(package, s);
		strcat(package, "/");
		#ifdef GIVE_FILE_INFO
		printf("%s \n", package);
		#endif
		return 0;
	case 2: if(s[0] == '$')   {
			state = 1;
			return 0;
		}
		else   {
			set_file_name(s);
			temp = allocate_mem(char, (strlen(package)+strlen(output_file) + 1));
			check_mem_alloc_error(temp);
			strcpy(temp, package);
			strcat(temp, output_file);
			free(output_file);
			output_file = temp;
		}
		break;
	default:
		print_error();
		exit(1);
	}
	return 1;
}

int parse_for_include_exclude_classes(int argc, char *argv[])   {
	int current_option = 1;
	
	if(argv[current_option][0] == '-' )   {
		while(1)   
		switch(argv[current_option][1])   {
		case 'I' : classes_include_flag = ON;
			   current_option++;
			   do   {
				classes_list = (char **)realloc(classes_list, sizeof(char *) * (number_of_classes + 1));
				classes_list[number_of_classes] = strdup(argv[current_option]);
				number_of_classes++;
				current_option++;
			   }while(argv[current_option][0] != '-');
			   break;
		case 'E' : classes_exclude_flag = ON;
			   current_option++;
			   do   {
				classes_list = (char **)realloc(classes_list, sizeof(char **) * (number_of_classes + 1));
				classes_list[number_of_classes] = strdup(argv[current_option]);
				number_of_classes++;
				current_option++;
			   }while(argv[current_option][0] != '-');
			   break;
		case 'F'  : 
			   return current_option;
		}
	}
	else   {
		fprintf(stderr, "usage: parser [File Name]+[$ Package Name file+]*\n");
		return -1;
	}
	return -1;
}
void create_package(char *s)   {
	int i = 0, j = 0, k = 0, dfd, sfd, length;
	char path[1024];
	struct stat * buf;
	while(s[i] != '\0')  {
		path[i]=s[i];
		if(s[i] == FILE_SEPARATOR)  {
			k = j;
			j = i + 1;
		}
		i++;
	}
	if( k == 0)
		k = j;
	path[k] = '\0';
	strcat(path, "templates/Test.java");
	mkdir ("pack", 0777);
	buf = allocate_one_element(struct stat);
	check_mem_alloc_error(buf);
	if(stat("pack/Test.java", buf) != -1)   {
		fprintf(stderr, "The file pack/Test.java already exists\n");
		exit(1);
	}
	free(buf);
	unlink("pack/Test.java");
	sfd = open(path, O_RDONLY);
	dfd = open("pack/Test.java", O_RDWR | O_CREAT, 0666);
	if( dfd == -1 )  {
		fprintf(stderr, "Sorry unable to create file \n");
		exit(1);
	}
	if( sfd == -1 )  {
		fprintf(stderr, "Sorry unable to open file %s\n", path);
		exit(1);
	}
	length = read(sfd, path, 1024);
	while(length > 0)  {
		write(dfd, path, length);
		length = read(sfd, path, 1024);
	}
	close(sfd);
	close(dfd);
}

int main(int argc,char *argv[])   {
	int i, j, temp;
	FILE *input_file;
	int yyparse();

	if(argc <= 1)   {
		fprintf(stderr, "Too Few arguments\n");
		fprintf(stderr, "usage: parser [-[I|E] Class Name +] [-F File Name]* [$ Package Name file+]*\n");
		return 1;
	}
	create_package(argv[0]);
	unlink("block_history.dat");
	close(open("block_history.dat", O_RDWR | O_CREAT, 0666 ));
	unlink("functions_history.dat");
	close(open("functions_history.dat", O_RDWR | O_CREAT, 0666 ));
	unlink("classes_history.dat");
	close(open("classes_history.dat", O_RDWR | O_CREAT, 0666 ));
	unlink("branches_history.dat");
	close(open("branches_history.dat", O_RDWR | O_CREAT, 0666 ));
	close(open("report.dat", O_RDWR | O_CREAT, 0666 ));
	close(open("report.html", O_RDWR | O_CREAT, 0666 ));
	
	temp = parse_for_include_exclude_classes(argc, argv); 
	if(temp == -1)
		return 1;
	argc = argc - temp;
	argv = argv + temp;
	
	file_names_list = allocate_mem(char *, (argc - 1));
	check_mem_alloc_error(file_names_list);
	file_lineno_list = allocate_mem(int *, (argc - 1));
	check_mem_alloc_error(file_lineno_list);
	lineno = 1;
	file_times_list = initialize();
	for( i = 1, j = 1; i < argc ; i++, j++)   {
		while(set_file_name1(argv[i]) == 0)
			i++;
		#ifdef GIVE_FILE_INFO
		printf("\t  %s\n", argv[i] );
		#endif
		index_after_import_statements = 0;
		class_list = initialize();
		func_list = initialize();
		stat_count_list = initialize();
		block_list = initialize();
		branch_cov_list = initialize();
		v_b_stack = initialize_brace_stack();
		if((input_file = fopen(argv[i], "r")) == NULL) {
			fprintf(stderr, "Error opening input file %s\n", argv[i]);
			exit(1);
		}
		file_times_list = insert_file_times_list(file_times_list, argv[i]);
		#ifdef DEBUG
		printf("The File name = %s lineno = %d \n", argv[i], __LINE__);
		#endif
		if(i == (argc - 1))   
			number_of_files = j + 1;
		input_file_name = argv[i];
		file_names_list[j - 1] = strdup(output_file);
		check_mem_alloc_error(file_names_list[j-1]);
		yyin = input_file;
		is_inside_class = 0;
		is_parsing_if = 0;
		current_file_number = j - 1;
		yyparse();
		#ifdef DEBUG
		print_branch_coverage_list(branch_cov_list);
		#endif
		file_lineno_list[j - 1] = allocate_one_element(int);
		file_lineno_list[j - 1][0] = lineno - 1;
	    	stat_count_list = reverse_list(stat_count_list);
	    	block_list = reverse_list(block_list);
	    	func_list = reverse_list(func_list);
		branch_cov_list = reverse_list(branch_cov_list);
		v_t_class_list = append(v_t_class_list, class_list);
		v_t_block_list = append(v_t_block_list, block_list);
		v_t_func_list = append(v_t_func_list, func_list);
		v_t_stat_count_list = append(v_t_stat_count_list, stat_count_list);
		v_t_branch_cov_list = append(v_t_branch_cov_list, branch_cov_list);
	    	instrument(func_list, fd);
	    	close(fd);
		base_number_of_class_list += lengthoflist(class_list);
	}
	file_lineno_list[j - 2][0] -= 1;
	generate_last_file();
	#ifdef DEBUG
	print_file_times_list(file_times_list); 
	#endif
	#ifdef COMMENT_LINENO_COUNT
	printf("The number of commenting lines are %d \n",comment_lineno_count);
	#endif
	return return_code;
}

int yywrap(){
	;
}

