#include "linkedlist.h"
/**
 * @Developed by N V Sreenivasa Rao
 * @Supervisor   Prof Pankaj Jalote
 * @URL          http://www.cse.iitk.ac.in/users/jalote/download/javacoverage/index.html
 * @title        JavaCoverage
 * @version      1.0
 */

linkedlist_ptr initialize(void)   {
	linkedlist_ptr temp;
	temp = allocate_one_element(linkedlist);
	check_mem_alloc_error(temp);
	temp->element = temp->next = NULL;
	return temp;
}

/* This function reverses the list
 */
linkedlist_ptr reverse_list(linkedlist_ptr l)   {
	typedef void ** stack;
	stack s;
	linkedlist_ptr l1 = l;
	int counter = 0, i;

	if(l == NULL)	{
		invalid_arg_error();
		exit(1);
	}
	while(l1 ->next != NULL)   {
		counter++;
		l1 = l1->next;
	}
	l1 = l;
	s = allocate_mem(void *, counter);
	for( i = 0; i < counter; i++)   {
		s[i] = l1->element;
		l1 = l1->next;
	}
	l1 = l;
	while(counter)   {
		counter--;
		l1->element = s[counter];
		l1 = l1->next;
	}
	free(s);
	return l;
}

/* This function inserts the element in the given list, here the ordering
 * is checked with the help of function 'cmp' which comes as an argument
 */
linkedlist_ptr insert(linkedlist_ptr l, void * element, int(*cmp)(void *, void *))   {
	linkedlist_ptr temp, temp1 = l, prev_temp;
	if(l == NULL || element == NULL || cmp == NULL)   {
		invalid_arg_error();
		exit(1);
	}
	temp = allocate_one_element(linkedlist);
	check_mem_alloc_error(temp);
	temp->element = element;
	temp->next = l;
	if(l->next == NULL || !(*cmp)(element, temp1->element))
		return temp;
	while(temp1->next != NULL && (*cmp)(element, temp1->element))  {
		prev_temp = temp1;
		temp1 = temp1->next;
	}
	temp->next = prev_temp->next;
	prev_temp->next = temp;
	return l;
}

/* This function returns the number of nodes present in the linked list
 */

int lengthoflist(linkedlist_ptr l)   {
	int counter = 0;
	while(l->next != NULL)   {
		counter++;
		l = l->next;
	}
	return counter;
}

linkedlist_ptr append(linkedlist_ptr dest, linkedlist_ptr src)  {
	linkedlist_ptr t_dest = dest, temp;
	if(dest == NULL)
		return src;
	if(dest->next == NULL)   {
		free(dest);
		return src;
	}
	while(t_dest->next->next != NULL)
		t_dest = t_dest->next;
	temp = t_dest->next;
	t_dest->next = src;
	return dest;
}

void printlist(linkedlist_ptr l, void (*print)(void *))   {
	if(l == NULL || print == NULL)   {
		invalid_arg_error();
		exit(1);
	}
	while(l->next != NULL)   {
		(*print)(l->element);
		l = l->next;
	}
	return;
}

/* This function rotates the list right by one element.
 */ 

linkedlist_ptr rotate_left(linkedlist_ptr l)   {
	linkedlist_ptr temp, temp1;
	if(l->next == NULL || l->next->next == NULL)
		return l;
	temp = l;
	temp1 = l->next;
	while(l->next->next != NULL)
		l = l->next;
	temp->next = l->next;
	l->next = temp;
	return temp1;
}

int indexof(linkedlist_ptr l, void * e)  {
	int counter = 0;
	if(l == NULL || e == NULL)   {
		invalid_arg_error();
		exit(1);
	}
	while(l->next != NULL)   {
		counter++;
		if(l->element == e)
			return counter;
		l = l->next;
	}
	return -1;
}
